<?php declare(strict_types=1);

/*
 *     Copyleft (C) 2017  http://sikofitt.com sikofitt@gmail.com
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Sikofitt\SamsungTV\Console\Command;

use Sikofitt\SamsungTV\Actions\Actions;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Exception\InvalidArgumentException;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;

/**
 * Command to change the input of the Samsung TV.
 */
class InputCommand extends Command
{
    /**
     * {@inheritDoc}
     *
     * @throws \Symfony\Component\Console\Exception\InvalidArgumentException
     */
    public function configure(): void
    {
        $this->setName('input')
            ->setDescription('Change the TV source')
            ->addOption('port', 'p', InputOption::VALUE_REQUIRED, 'Port of the Samsung TV', 55000)
            ->addOption('address', 'a', InputOption::VALUE_REQUIRED, 'IP or Host of the SamsungTV')
            ->addArgument(
                'input',
                InputArgument::REQUIRED,
                'The input channel. For a list type \'list\'.'
            );
    }

    /**
     * {@inheritDoc}
     *
     * @throws \Symfony\Component\Console\Exception\InvalidArgumentException
     */
    public function interact(InputInterface $input, OutputInterface $output): void
    {
        $inputArg = $input->getArgument('input');

        if ('list' === strtolower($inputArg)) {
            $output->writeln(sprintf('Available inputs are : %s.%s', implode(', ', $this->getAvailableInputs()), PHP_EOL));
            exit;
        }

        if(null === $input->getOption('address')) {
            throw new InvalidArgumentException('We need an ip address or host to connect to.');
        }

        if (false === in_array(strtolower($inputArg), $this->getAvailableInputs(), true)) {
            throw new InvalidArgumentException('Input ' . $inputArg . ' is not available');
        }
    }

    /**
     * {@inheritDoc}
     *
     * @throws \Symfony\Component\Console\Exception\InvalidArgumentException
     *
     * @return null|int
     */
    public function execute(InputInterface $input, OutputInterface $output): ?int
    {
        $io = new SymfonyStyle($input, $output);
        $inputArg = $input->getArgument('input');

        try {
            $actions = new Actions($input->getOption('address'), $input->getOption('port'));
            $actions->changeInput(strtoupper($inputArg));
        } catch (\Exception $e) {
            $io->error($e->getMessage());
            return 1;
        } finally {
            $io->success('Changed source input to ' . $inputArg);
            return 0;
        }
    }

    /**
     * @return array
     */
    private function getAvailableInputs(): array
    {
        return [
          'tv',
          'hdmi',
          'hdmi1', // different than hdmi for some reason.
          'hdmi2',
          'hdmi3',
          'hdmi4',
          'av1',
          'av2',
          'av3',
        ];
    }
}
